package com.josh.tiny;
// Nice little fixed-width Vga Font


import java.awt.Font;
import java.awt.Graphics;
import java.awt.font.FontRenderContext;

import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.*;
import java.awt.*;


public class VgaSoftFont {
	

	public int height;
	public int width;
	public int count;
	
	public boolean pixels[][][];
	
	public int[] colors = new int[16];
	
	//Read a line, skipping lines that start with # for comments
	
	private String rl(BufferedReader br) throws IOException {
		
		String s;
		
		do {
			s= br.readLine();
			
		} while ( s.charAt(0)=='#' || s.charAt(0)==';');
		
		return(s);
	
	}
			
		
	private void loadFromTextFile( File fontfile ) throws IOException {
		
		BufferedReader input = new BufferedReader(new FileReader(fontfile ));
		
		String s;
		
			
		s = rl(input);
		
		
		height = Integer.parseInt(s);
		
		s = rl(input);
		width = Integer.parseInt(s);	
		
		for( int cl = 0 ; cl < 16 ; cl++ ){
			
			s = rl(input);
			
			colors[cl] = Integer.parseInt(s,16);
			
		}
		
		count = 256;
			
		pixels = new boolean[count][height][width];
		
		for( int cl = 0 ; cl < count ; cl++) {
			
			for( int yl = 0 ; yl < height ; yl++) {
				
				s = rl(input);
				
				for ( int xl = 0 ; xl < width ; xl++ ) {
					
					if ( "xX*oO".indexOf( s.charAt(xl) ) > 0) {	
						
						pixels[cl][yl][xl]= true;
						
					}						
			
				}
				
			}
			
			
		}	
		
		input.close();
			
	}
	
	// This just loads a single font file I found on the internet for a nice 5x7 font
	
	private void loadFromDBFile( File fontfile ) throws IOException {
		
		count  = 256;
		height = 8;
		width  = 6;
		
		pixels = new boolean[count][height][width];
				
		BufferedReader input = new BufferedReader(new FileReader(fontfile ));
		
		String s;		
		
		for( int c = 0 ; c < 256 ; c++ ) {
			
			s = rl(input);
			
			if (s==null ) break;
			
			String b[] = s.split("[\\s\\t,h]+");
			
			System.out.println("-"+s+"-");
			
			int bits[] = new int[6];
			
			for( int l=0 ; l < 6 ; l++ ) {
				
				bits[l] = Integer.parseInt( b[l+2], 16);
				
				System.out.println(l+"="+Integer.toBinaryString(bits[l]));
			}
			
			int mask = 0x01;
			
			for( int y = 0 ; y < 8 ; y++ ) {
				
				for( int x = 0; x < 6 ; x++ ) {
				
					if ( (bits[x] & mask )!=0) {
						
						pixels[c][y][x] = true;
					} else {
						pixels[c][y][x] = false;

												
					} 
				}
				
				mask *= 2;

			}
			
				
				
		}
					
		input.close();
		
		colors = defaultColors;
		
		
	}	
	
	
	private void loadFromTTFFile( File file ) throws IOException, FontFormatException {
				
		Font font = Font.createFont( Font.TRUETYPE_FONT ,  file );
		
		System.out.println("loaded "+ font.getFamily() );
		
		
		font = font.deriveFont((float) 23 );
		
		// First figure out how big the biggest char is
				
		FontRenderContext frc = new FontRenderContext( null , false , false );
		
		Rectangle2D r= font.getMaxCharBounds(frc);
		
		count = 256;
		
		width = (int) Math.ceil( r.getWidth() );
		height = (int) Math.ceil( r.getHeight() );
		
		pixels = new boolean[count][height][width];
		
		
		BufferedImage bufferedImage = new BufferedImage(  width , height , BufferedImage.TYPE_BYTE_BINARY  ) ;
		
		byte[] b = { (byte) 65 };
			
		Graphics g = bufferedImage.getGraphics();
		
		g.setFont(font);
		g.setColor(Color.WHITE);
		
		
		for( int c=0; c <256; c++ ) {
			
			if ( font.canDisplay((char) c )) {
			
				g.clearRect( 0 , 0 , width , height );
				
				b[0] = (byte) c;
				
				int startline =  height - g.getFontMetrics().getDescent();				
							
				g.drawBytes( b , 0, 1 , 0 , startline );
				
				for( int y=0; y< height; y++ ){
					
					for( int x=0; x <width ;x++) {
							
						if ( (bufferedImage.getRGB(x, y) & 0x00ffffff) != 0 ) {						
							pixels[c][y][x] = true;
						} else {
							pixels[c][y][x] = false;
	
						}
						
					}

				}
			}
		}
		
		colors = defaultColors;
		
	}
	
	//Doesn't work right now
	
	/*
	
	private void loadFromDumpFontFile( File fontfile ) throws IOException {
		
		count  = 256;
		height = 8;
		width  = 6;
		
		pixels = new boolean[count][height][width];
			
		
		BufferedReader input = new BufferedReader(new FileReader(fontfile ));
		
		String s;
		
		while (true) {
			
			s = rl(input);
			height = Integer.parseInt(s);
			
			s = rl(input);
			width = Integer.parseInt(s);	
			
			for( int cl = 0 ; cl < 16 ; cl++ ){
				
				s = rl(input);
				
				colors[cl] = Integer.parseInt(s,16);
				
			}
			
			count = 256;
				
			pixels = new boolean[count][height][width];
			
			for( int cl = 0 ; cl < count ; cl++) {
				
				for( int yl = 0 ; yl < height ; yl++) {
					
					s = rl(input);
					
					for ( int xl = 0 ; xl < width ; xl++ ) {
						
						if ( "xX*oO".indexOf( s.charAt(xl) ) > 0) {	
							
							pixels[cl][yl][xl]= true;
							
						}						
				
					}
					
				}
				
				
			}	
			
		}
		
		colors = defaultColors;	
		
		
	}	
	
	*/
	
	// Reads in a file created by FONTEXE.EXE but only works for standard 16 line fonts
	
	private void loadFromFontEditFile( File file ) throws IOException {
		
		count  = 256;
		height = 16;
		width  = 8;
		
		pixels = new boolean[count][height][width];
			
		byte[] inLine = new byte[1];		// One line of pixels

		StringBuffer outLine = new StringBuffer(8);
		
		RandomAccessFile f = new RandomAccessFile( file , "r" );
				
		f.seek( 0x163 );		// Jump to the start of the font data
		
		int z = 16;			// not sure why but fontfile starts at ASCII 16 and then wraps
		
		for( int c = 0 ; c < 256 ; c++ ) {
								
			for( int y=0; y < 16 ; y++ ) {
				
				f.read( inLine );
	
				outLine.setLength(0);
				
				for( int x = 0; x < 8 ; x++ ) {
					
					int b = inLine[0];		// The byte
					
					int m = 128 >> x ;		// The mask
					
					if ( (b & m) != 0 ) {
						
						pixels[z][y][x] = true;
						
						
					} else {
						
						pixels[z][y][x] = false;
						
					}
					
				}
				

				
			}
			
			z++;
			
			if (z==256) z=0;
					
			
		}
		
		f.close();
		
		colors = defaultColors;
		
	}
	
	public VgaSoftFont( File fontfile ) throws IOException {
		
		if (fontfile==null) {
			
			loadDefaultFont();
			
		} else {
		
			loadFromTextFile(fontfile);
			
		}

	}
	
	public VgaSoftFont() {
				
		loadDefaultFont();
	
	}
	
	static enum FontFileType { Text, DBFont , FontEdit , TTF };
	
	public VgaSoftFont( File fontfile , FontFileType fontFileType ) throws FileNotFoundException, IOException, FontFormatException {
		
		switch(fontFileType){
		
				case Text:{
					loadFromTextFile( fontfile );
				};
				break;
				
				case DBFont: {
					
					loadFromDBFile(fontfile);
					
					
				};
				break;
				
				case FontEdit: {
					
					loadFromFontEditFile(fontfile);
					
				};
				break;
			
				case TTF: {
					
					loadFromTTFFile(fontfile);
					
				};
				break;
		}
		
	}
	
	// Load the default font
	
	
	
	
	private void loadDefaultFont() {

		final String cells[][] =	{

	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        " XXXXXX ",
	        "X      X",
	        "X X  X X",
	        "X      X",
	        "X      X",
	        "X XXXX X",
	        "X  XX  X",
	        "X      X",
	        "X      X",
	        " XXXXXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        " XXXXXX ",
	        "XXXXXXXX",
	        "XX XX XX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XX    XX",
	        "XXX  XXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        " XXXXXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        " XX XX  ",
	        "XXXXXXX ",
	        "XXXXXXX ",
	        "XXXXXXX ",
	        "XXXXXXX ",
	        " XXXXX  ",
	        "  XXX   ",
	        "   X    ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "   X    ",
	        "  XXX   ",
	        " XXXXX  ",
	        "XXXXXXX ",
	        " XXXXX  ",
	        "  XXX   ",
	        "   X    ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "   XX   ",
	        "  XXXX  ",
	        "  XXXX  ",
	        "XXX  XXX",
	        "XXX  XXX",
	        "XXX  XXX",
	        "X  XX  X",
	        "   XX   ",
	        "  XXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "   XX   ",
	        "  XXXX  ",
	        " XXXXXX ",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        " XXXXXX ",
	        "   XX   ",
	        "   XX   ",
	        "  XXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "   XX   ",
	        "  XXXX  ",
	        "  XXXX  ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXX  XXX",
	        "XX    XX",
	        "XX    XX",
	        "XXX  XXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "  XXXX  ",
	        " XX  XX ",
	        " X    X ",
	        " X    X ",
	        " XX  XX ",
	        "  XXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XX    XX",
	        "X  XX  X",
	        "X XXXX X",
	        "X XXXX X",
	        "X  XX  X",
	        "XX    XX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	    },
	    {
	        "        ",
	        "        ",
	        "   XXXX ",
	        "    XXX ",
	        "   XX X ",
	        "  XX  X ",
	        " XXXX   ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        " XXXX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "  XXXX  ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        "  XXXX  ",
	        "   XX   ",
	        " XXXXXX ",
	        "   XX   ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "  XXXXXX",
	        "  XX  XX",
	        "  XXXXXX",
	        "  XX    ",
	        "  XX    ",
	        "  XX    ",
	        "  XX    ",
	        " XXX    ",
	        "XXXX    ",
	        "XXX     ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        " XXXXXXX",
	        " XX   XX",
	        " XXXXXXX",
	        " XX   XX",
	        " XX   XX",
	        " XX   XX",
	        " XX   XX",
	        " XX  XXX",
	        "XXX  XXX",
	        "XXX  XX ",
	        "XX      ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "   XX   ",
	        "   XX   ",
	        "XX XX XX",
	        "  XXXX  ",
	        "XXX  XXX",
	        "  XXXX  ",
	        "XX XX XX",
	        "   XX   ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        "
	    },
		
			
		{
	        "        ",
	        "X       ",
	        "XX      ",
	        "XXX     ",
	        "XXXX    ",
	        "XXXXX   ",
	        "XXXXXXX ",
	        "XXXXX   ",
	        "XXXX    ",
	        "XXX     ",
	        "XX      ",
	        "X       ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "      X ",
	        "     XX ",
	        "    XXX ",
	        "   XXXX ",
	        "  XXXXX ",
	        "XXXXXXX ",
	        "  XXXXX ",
	        "   XXXX ",
	        "    XXX ",
	        "     XX ",
	        "      X ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "   XX   ",
	        "  XXXX  ",
	        " XXXXXX ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        " XXXXXX ",
	        "  XXXX  ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        "        ",
	        " XX  XX ",
	        " XX  XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        " XXXXXXX",
	        "XX XX XX",
	        "XX XX XX",
	        "XX XX XX",
	        " XXXX XX",
	        "   XX XX",
	        "   XX XX",
	        "   XX XX",
	        "   XX XX",
	        "   XX XX",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        " XXXXX  ",
	        "XX   XX ",
	        " XX     ",
	        "  XXX   ",
	        " XX XX  ",
	        "XX   XX ",
	        "XX   XX ",
	        " XX XX  ",
	        "  XXX   ",
	        "    XX  ",
	        "XX   XX ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XXXXXXX ",
	        "XXXXXXX ",
	        "XXXXXXX ",
	        "XXXXXXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "   XX   ",
	        "  XXXX  ",
	        " XXXXXX ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        " XXXXXX ",
	        "  XXXX  ",
	        "   XX   ",
	        " XXXXXX ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "   XX   ",
	        "  XXXX  ",
	        " XXXXXX ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        " XXXXXX ",
	        "  XXXX  ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "   XX   ",
	        "    XX  ",
	        "XXXXXXX ",
	        "    XX  ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "  XX    ",
	        " XX     ",
	        "XXXXXXX ",
	        " XX     ",
	        "  XX    ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XX      ",
	        "XX      ",
	        "XX      ",
	        "XX      ",
	        "XXXXXXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "  X  X  ",
	        " XX  XX ",
	        "XXXXXXXX",
	        " XX  XX ",
	        "  X  X  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "   X    ",
	        "  XXX   ",
	        "  XXX   ",
	        " XXXXX  ",
	        " XXXXX  ",
	        "XXXXXXX ",
	        "XXXXXXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XXXXXXX ",
	        "XXXXXXX ",
	        " XXXXX  ",
	        " XXXXX  ",
	        "  XXX   ",
	        "  XXX   ",
	        "   X    ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "   XX   ",
	        "  XXXX  ",
	        "  XXXX  ",
	        "  XXXX  ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "        ",
	        "   XX   ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        "  X  X  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        " XX XX  ",
	        " XX XX  ",
	        "XXXXXXX ",
	        " XX XX  ",
	        " XX XX  ",
	        " XX XX  ",
	        "XXXXXXX ",
	        " XX XX  ",
	        " XX XX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "   XX   ",
	        "   XX   ",
	        " XXXXX  ",
	        "XX   XX ",
	        "XX    X ",
	        "XX      ",
	        " XXXXX  ",
	        "     XX ",
	        "X    XX ",
	        "XX   XX ",
	        " XXXXX  ",
	        "   XX   ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XX    X ",
	        "XX   XX ",
	        "    XX  ",
	        "   XX   ",
	        "  XX    ",
	        " XX     ",
	        "XX   XX ",
	        "X    XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "  XXX   ",
	        " XX XX  ",
	        " XX XX  ",
	        "  XXX   ",
	        " XXX XX ",
	        "XX XXX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        " XXX XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "  XX    ",
	        "  XX    ",
	        "  XX    ",
	        " XX     ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "    XX  ",
	        "   XX   ",
	        "  XX    ",
	        "  XX    ",
	        "  XX    ",
	        "  XX    ",
	        "  XX    ",
	        "  XX    ",
	        "   XX   ",
	        "    XX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "  XX    ",
	        "   XX   ",
	        "    XX  ",
	        "    XX  ",
	        "    XX  ",
	        "    XX  ",
	        "    XX  ",
	        "    XX  ",
	        "   XX   ",
	        "  XX    ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        " XX  XX ",
	        "  XXXX  ",
	        "XXXXXXXX",
	        "  XXXX  ",
	        " XX  XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "   XX   ",
	        "   XX   ",
	        " XXXXXX ",
	        "   XX   ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "  XX    ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XXXXXXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "   XX   ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "      X ",
	        "     XX ",
	        "    XX  ",
	        "   XX   ",
	        "  XX    ",
	        " XX     ",
	        "XX      ",
	        "X       ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        " XXXXX  ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX X XX ",
	        "XX X XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "   XX   ",
	        "  XXX   ",
	        " XXXX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        " XXXXXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        " XXXXX  ",
	        "XX   XX ",
	        "     XX ",
	        "    XX  ",
	        "   XX   ",
	        "  XX    ",
	        " XX     ",
	        "XX      ",
	        "XX   XX ",
	        "XXXXXXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        " XXXXX  ",
	        "XX   XX ",
	        "     XX ",
	        "     XX ",
	        "  XXXX  ",
	        "     XX ",
	        "     XX ",
	        "     XX ",
	        "XX   XX ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "    XX  ",
	        "   XXX  ",
	        "  XXXX  ",
	        " XX XX  ",
	        "XX  XX  ",
	        "XXXXXXX ",
	        "    XX  ",
	        "    XX  ",
	        "    XX  ",
	        "   XXXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XXXXXXX ",
	        "XX      ",
	        "XX      ",
	        "XX      ",
	        "XXXXXX  ",
	        "    XXX ",
	        "     XX ",
	        "     XX ",
	        "XX   XX ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "  XXX   ",
	        " XX     ",
	        "XX      ",
	        "XX      ",
	        "XXXXXX  ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XXXXXXX ",
	        "XX   XX ",
	        "     XX ",
	        "     XX ",
	        "    XX  ",
	        "   XX   ",
	        "  XX    ",
	        "  XX    ",
	        "  XX    ",
	        "  XX    ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        " XXXXX  ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        " XXXXX  ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        " XXXXX  ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        " XXXXXX ",
	        "     XX ",
	        "     XX ",
	        "     XX ",
	        "    XX  ",
	        " XXXX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "   XX   ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "        ",
	        "   XX   ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "   XX   ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "        ",
	        "   XX   ",
	        "   XX   ",
	        "  XX    ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "     XX ",
	        "    XX  ",
	        "   XX   ",
	        "  XX    ",
	        " XX     ",
	        "  XX    ",
	        "   XX   ",
	        "    XX  ",
	        "     XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XXXXXXX ",
	        "        ",
	        "        ",
	        "XXXXXXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        " XX     ",
	        "  XX    ",
	        "   XX   ",
	        "    XX  ",
	        "     XX ",
	        "    XX  ",
	        "   XX   ",
	        "  XX    ",
	        " XX     ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        " XXXXX  ",
	        "XX   XX ",
	        "XX   XX ",
	        "    XX  ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "        ",
	        "   XX   ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        " XXXXX  ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX XXXX ",
	        "XX XXXX ",
	        "XX XXXX ",
	        "XX XXX  ",
	        "XX      ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "   X    ",
	        "  XXX   ",
	        " XX XX  ",
	        "XX   XX ",
	        "XX   XX ",
	        "XXXXXXX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XXXXXX  ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XXXXX  ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        "XXXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "  XXXX  ",
	        " XX  XX ",
	        "XX    X ",
	        "XX      ",
	        "XX      ",
	        "XX      ",
	        "XX      ",
	        "XX    X ",
	        " XX  XX ",
	        "  XXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XXXXX   ",
	        " XX XX  ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX XX  ",
	        "XXXXX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XXXXXXX ",
	        " XX  XX ",
	        " XX   X ",
	        " XX X   ",
	        " XXXX   ",
	        " XX X   ",
	        " XX     ",
	        " XX   X ",
	        " XX  XX ",
	        "XXXXXXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XXXXXXX ",
	        " XX  XX ",
	        " XX   X ",
	        " XX X   ",
	        " XXXX   ",
	        " XX X   ",
	        " XX     ",
	        " XX     ",
	        " XX     ",
	        "XXXX    ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "  XXXX  ",
	        " XX  XX ",
	        "XX    X ",
	        "XX      ",
	        "XX      ",
	        "XX XXXX ",
	        "XX   XX ",
	        "XX   XX ",
	        " XX  XX ",
	        "  XXX X ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XXXXXXX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "  XXXX  ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "  XXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "   XXXX ",
	        "    XX  ",
	        "    XX  ",
	        "    XX  ",
	        "    XX  ",
	        "    XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        " XXXX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XXX  XX ",
	        " XX  XX ",
	        " XX XX  ",
	        " XX XX  ",
	        " XXXX   ",
	        " XXXX   ",
	        " XX XX  ",
	        " XX  XX ",
	        " XX  XX ",
	        "XXX  XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XXXX    ",
	        " XX     ",
	        " XX     ",
	        " XX     ",
	        " XX     ",
	        " XX     ",
	        " XX     ",
	        " XX   X ",
	        " XX  XX ",
	        "XXXXXXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XX   XX ",
	        "XXX XXX ",
	        "XXXXXXX ",
	        "XXXXXXX ",
	        "XX X XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XX   XX ",
	        "XXX  XX ",
	        "XXXX XX ",
	        "XXXXXXX ",
	        "XX XXXX ",
	        "XX  XXX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "  XXX   ",
	        " XX XX  ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        " XX XX  ",
	        "  XXX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XXXXXX  ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XXXXX  ",
	        " XX     ",
	        " XX     ",
	        " XX     ",
	        " XX     ",
	        "XXXX    ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        " XXXXX  ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX X XX ",
	        "XX XXXX ",
	        " XXXXX  ",
	        "    XX  ",
	        "    XXX ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XXXXXX  ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XXXXX  ",
	        " XX XX  ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        "XXX  XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        " XXXXX  ",
	        "XX   XX ",
	        "XX   XX ",
	        " XX     ",
	        "  XXX   ",
	        "    XX  ",
	        "     XX ",
	        "XX   XX ",
	        "XX   XX ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        " XXXXXX ",
	        " XXXXXX ",
	        " X XX X ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "  XXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        " XX XX  ",
	        "  XXX   ",
	        "   X    ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX X XX ",
	        "XX X XX ",
	        "XXXXXXX ",
	        " XX XX  ",
	        " XX XX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XX   XX ",
	        "XX   XX ",
	        " XX XX  ",
	        " XX XX  ",
	        "  XXX   ",
	        "  XXX   ",
	        " XX XX  ",
	        " XX XX  ",
	        "XX   XX ",
	        "XX   XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        "  XXXX  ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "  XXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XXXXXXX ",
	        "XX   XX ",
	        "X    XX ",
	        "    XX  ",
	        "   XX   ",
	        "  XX    ",
	        " XX     ",
	        "XX    X ",
	        "XX   XX ",
	        "XXXXXXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "  XXXX  ",
	        "  XX    ",
	        "  XX    ",
	        "  XX    ",
	        "  XX    ",
	        "  XX    ",
	        "  XX    ",
	        "  XX    ",
	        "  XX    ",
	        "  XXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "X       ",
	        "XX      ",
	        "XXX     ",
	        " XXX    ",
	        "  XXX   ",
	        "   XXX  ",
	        "    XXX ",
	        "     XX ",
	        "      X ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "  XXXX  ",
	        "    XX  ",
	        "    XX  ",
	        "    XX  ",
	        "    XX  ",
	        "    XX  ",
	        "    XX  ",
	        "    XX  ",
	        "    XX  ",
	        "  XXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "   X    ",
	        "  XXX   ",
	        " XX XX  ",
	        "XX   XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XXXXXXXX",
	        "        ",
	        "        ",
	    },
	    {
	        "  XX    ",
	        "  XX    ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        " XXXX   ",
	        "    XX  ",
	        " XXXXX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        " XXX XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XXX     ",
	        " XX     ",
	        " XX     ",
	        " XXXX   ",
	        " XX XX  ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        "XX XXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        " XXXXX  ",
	        "XX   XX ",
	        "XX      ",
	        "XX      ",
	        "XX      ",
	        "XX   XX ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "   XXX  ",
	        "    XX  ",
	        "    XX  ",
	        "  XXXX  ",
	        " XX XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        " XXX XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        " XXXXX  ",
	        "XX   XX ",
	        "XXXXXXX ",
	        "XX      ",
	        "XX      ",
	        "XX   XX ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "  XXX   ",
	        " XX XX  ",
	        " XX  X  ",
	        " XX     ",
	        "XXXX    ",
	        " XX     ",
	        " XX     ",
	        " XX     ",
	        " XX     ",
	        "XXXX    ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        " XXX XX ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        " XXXXX  ",
	        "    XX  ",
	        "XX  XX  ",
	        " XXXX   ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XXX     ",
	        " XX     ",
	        " XX     ",
	        " XX XX  ",
	        " XXX XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        "XXX  XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "   XX   ",
	        "   XX   ",
	        "        ",
	        "  XXX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "  XXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "     XX ",
	        "     XX ",
	        "        ",
	        "    XXX ",
	        "     XX ",
	        "     XX ",
	        "     XX ",
	        "     XX ",
	        "     XX ",
	        "     XX ",
	        " XX  XX ",
	        " XX  XX ",
	        "  XXXX  ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XXX     ",
	        " XX     ",
	        " XX     ",
	        " XX  XX ",
	        " XX XX  ",
	        " XXXX   ",
	        " XXXX   ",
	        " XX XX  ",
	        " XX  XX ",
	        "XXX  XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "  XXX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "  XXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XXX XX  ",
	        "XXXXXXX ",
	        "XX X XX ",
	        "XX X XX ",
	        "XX X XX ",
	        "XX X XX ",
	        "XX X XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XX XXX  ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        " XXXXX  ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XX XXX  ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XXXXX  ",
	        " XX     ",
	        " XX     ",
	        "XXXX    ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        " XXX XX ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        " XXXXX  ",
	        "    XX  ",
	        "    XX  ",
	        "   XXXX ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XX XXX  ",
	        " XXX XX ",
	        " XX   X ",
	        " XX     ",
	        " XX     ",
	        " XX     ",
	        "XXXX    ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        " XXXXX  ",
	        "XX   XX ",
	        " XX     ",
	        "  XXX   ",
	        "    XX  ",
	        "XX   XX ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "   X    ",
	        "  XX    ",
	        "  XX    ",
	        "XXXXXX  ",
	        "  XX    ",
	        "  XX    ",
	        "  XX    ",
	        "  XX    ",
	        "  XX XX ",
	        "   XXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        " XXX XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        "  XXXX  ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX X XX ",
	        "XX X XX ",
	        "XXXXXXX ",
	        " XX XX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XX   XX ",
	        " XX XX  ",
	        "  XXX   ",
	        "  XXX   ",
	        "  XXX   ",
	        " XX XX  ",
	        "XX   XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        " XXXXXX ",
	        "     XX ",
	        "    XX  ",
	        "XXXXX   ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XXXXXXX ",
	        "XX  XX  ",
	        "   XX   ",
	        "  XX    ",
	        " XX     ",
	        "XX   XX ",
	        "XXXXXXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "    XXX ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        " XXX    ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "    XXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "        ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        " XXX    ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "    XXX ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        " XXX    ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        " XXX XX ",
	        "XX XXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "   X    ",
	        "  XXX   ",
	        " XX XX  ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XXXXXXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "  XXXX  ",
	        " XX  XX ",
	        "XX    X ",
	        "XX      ",
	        "XX      ",
	        "XX      ",
	        "XX    X ",
	        " XX  XX ",
	        "  XXXX  ",
	        "    XX  ",
	        "     XX ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "        ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        " XXX XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "    XX  ",
	        "   XX   ",
	        "  XX    ",
	        "        ",
	        " XXXXX  ",
	        "XX   XX ",
	        "XXXXXXX ",
	        "XX      ",
	        "XX      ",
	        "XX   XX ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "   X    ",
	        "  XXX   ",
	        " XX XX  ",
	        "        ",
	        " XXXX   ",
	        "    XX  ",
	        " XXXXX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        " XXX XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "        ",
	        " XXXX   ",
	        "    XX  ",
	        " XXXXX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        " XXX XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        " XX     ",
	        "  XX    ",
	        "   XX   ",
	        "        ",
	        " XXXX   ",
	        "    XX  ",
	        " XXXXX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        " XXX XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "  XXX   ",
	        " XX XX  ",
	        "  XXX   ",
	        "        ",
	        " XXXX   ",
	        "    XX  ",
	        " XXXXX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        " XXX XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "  XXXX  ",
	        " XX  XX ",
	        " XX     ",
	        " XX     ",
	        " XX  XX ",
	        "  XXXX  ",
	        "    XX  ",
	        "     XX ",
	        "  XXXX  ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "   X    ",
	        "  XXX   ",
	        " XX XX  ",
	        "        ",
	        " XXXXX  ",
	        "XX   XX ",
	        "XXXXXXX ",
	        "XX      ",
	        "XX      ",
	        "XX   XX ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XX   XX ",
	        "XX   XX ",
	        "        ",
	        " XXXXX  ",
	        "XX   XX ",
	        "XXXXXXX ",
	        "XX      ",
	        "XX      ",
	        "XX   XX ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        " XX     ",
	        "  XX    ",
	        "   XX   ",
	        "        ",
	        " XXXXX  ",
	        "XX   XX ",
	        "XXXXXXX ",
	        "XX      ",
	        "XX      ",
	        "XX   XX ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        " XX  XX ",
	        " XX  XX ",
	        "        ",
	        "  XXX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "  XXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "   XX   ",
	        "  XXXX  ",
	        " XX  XX ",
	        "        ",
	        "  XXX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "  XXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        " XX     ",
	        "  XX    ",
	        "   XX   ",
	        "        ",
	        "  XXX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "  XXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "XX   XX ",
	        "XX   XX ",
	        "   X    ",
	        "  XXX   ",
	        " XX XX  ",
	        "XX   XX ",
	        "XX   XX ",
	        "XXXXXXX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "  XXX   ",
	        " XX XX  ",
	        "  XXX   ",
	        "        ",
	        "  XXX   ",
	        " XX XX  ",
	        "XX   XX ",
	        "XX   XX ",
	        "XXXXXXX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "   XX   ",
	        "  XX    ",
	        " XX     ",
	        "        ",
	        "XXXXXXX ",
	        " XX  XX ",
	        " XX     ",
	        " XXXXX  ",
	        " XX     ",
	        " XX     ",
	        " XX  XX ",
	        "XXXXXXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XX  XX  ",
	        " XXX XX ",
	        "  XX XX ",
	        " XXXXXX ",
	        "XX XX   ",
	        "XX XX   ",
	        " XX XXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "  XXXXX ",
	        " XX XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XXXXXXX ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "   X    ",
	        "  XXX   ",
	        " XX XX  ",
	        "        ",
	        " XXXXX  ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XX   XX ",
	        "XX   XX ",
	        "        ",
	        " XXXXX  ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        " XX     ",
	        "  XX    ",
	        "   XX   ",
	        "        ",
	        " XXXXX  ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "  XX    ",
	        " XXXX   ",
	        "XX  XX  ",
	        "        ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        " XXX XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        " XX     ",
	        "  XX    ",
	        "   XX   ",
	        "        ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        " XXX XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XX   XX ",
	        "XX   XX ",
	        "        ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        " XXXXXX ",
	        "     XX ",
	        "    XX  ",
	        " XXXX   ",
	        "        ",
	    },
	    {
	        "        ",
	        "XX   XX ",
	        "XX   XX ",
	        "        ",
	        "  XXX   ",
	        " XX XX  ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        " XX XX  ",
	        "  XXX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "XX   XX ",
	        "XX   XX ",
	        "        ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "   XX   ",
	        "   XX   ",
	        "  XXXX  ",
	        " XX  XX ",
	        " XX     ",
	        " XX     ",
	        " XX     ",
	        " XX  XX ",
	        "  XXXX  ",
	        "   XX   ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "  XXX   ",
	        " XX XX  ",
	        " XX  X  ",
	        " XX     ",
	        "XXXX    ",
	        " XX     ",
	        " XX     ",
	        " XX     ",
	        " XX     ",
	        "XXX  XX ",
	        "XXXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        " XX  XX ",
	        " XX  XX ",
	        "  XXXX  ",
	        "   XX   ",
	        " XXXXXX ",
	        "   XX   ",
	        " XXXXXX ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "XXXXX   ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XXXXX   ",
	        "XX   X  ",
	        "XX  XX  ",
	        "XX XXXX ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX   XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "    XXX ",
	        "   XX XX",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        " XXXXXX ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "XX XX   ",
	        " XXX    ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "   XX   ",
	        "  XX    ",
	        " XX     ",
	        "        ",
	        " XXXX   ",
	        "    XX  ",
	        " XXXXX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        " XXX XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "    XX  ",
	        "   XX   ",
	        "  XX    ",
	        "        ",
	        "  XXX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "  XXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "   XX   ",
	        "  XX    ",
	        " XX     ",
	        "        ",
	        " XXXXX  ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "   XX   ",
	        "  XX    ",
	        " XX     ",
	        "        ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        "XX  XX  ",
	        " XXX XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        " XXX XX ",
	        "XX XXX  ",
	        "        ",
	        "XX XXX  ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        " XXX XX ",
	        "XX XXX  ",
	        "        ",
	        "XX   XX ",
	        "XXX  XX ",
	        "XXXX XX ",
	        "XXXXXXX ",
	        "XX XXXX ",
	        "XX  XXX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "  XXXX  ",
	        " XX XX  ",
	        " XX XX  ",
	        "  XXXXX ",
	        "        ",
	        " XXXXXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "  XXX   ",
	        " XX XX  ",
	        " XX XX  ",
	        "  XXX   ",
	        "        ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "  XX    ",
	        "  XX    ",
	        "        ",
	        "  XX    ",
	        "  XX    ",
	        " XX     ",
	        "XX      ",
	        "XX   XX ",
	        "XX   XX ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XXXXXXX ",
	        "XX      ",
	        "XX      ",
	        "XX      ",
	        "XX      ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XXXXXXX ",
	        "     XX ",
	        "     XX ",
	        "     XX ",
	        "     XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "XX      ",
	        "XX      ",
	        "XX    X ",
	        "XX   XX ",
	        "XX  XX  ",
	        "   XX   ",
	        "  XX    ",
	        " XX     ",
	        "XX  XXX ",
	        "X  X  XX",
	        "     XX ",
	        "    XX  ",
	        "   XXXXX",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "XX      ",
	        "XX      ",
	        "XX    X ",
	        "XX   XX ",
	        "XX  XX  ",
	        "   XX   ",
	        "  XX    ",
	        " XX  XX ",
	        "XX  XXX ",
	        "X  XX X ",
	        "  XXXXXX",
	        "     XX ",
	        "    XXXX",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "   XX   ",
	        "   XX   ",
	        "        ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "  XXXX  ",
	        "  XXXX  ",
	        "  XXXX  ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "  XX  XX",
	        " XX  XX ",
	        "XX  XX  ",
	        " XX  XX ",
	        "  XX  XX",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XX  XX  ",
	        " XX  XX ",
	        "  XX  XX",
	        " XX  XX ",
	        "XX  XX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "   X   X",
	        " X   X  ",
	        "   X   X",
	        " X   X  ",
	        "   X   X",
	        " X   X  ",
	        "   X   X",
	        " X   X  ",
	        "   X   X",
	        " X   X  ",
	        "   X   X",
	        " X   X  ",
	        "   X   X",
	        " X   X  ",
	        "   X   X",
	        " X   X  ",
	    },
	    {
	        " X X X X",
	        "X X X X ",
	        " X X X X",
	        "X X X X ",
	        " X X X X",
	        "X X X X ",
	        " X X X X",
	        "X X X X ",
	        " X X X X",
	        "X X X X ",
	        " X X X X",
	        "X X X X ",
	        " X X X X",
	        "X X X X ",
	        " X X X X",
	        "X X X X ",
	    },
	    {
	        "XX XXX X",
	        " XXX XXX",
	        "XX XXX X",
	        " XXX XXX",
	        "XX XXX X",
	        " XXX XXX",
	        "XX XXX X",
	        " XXX XXX",
	        "XX XXX X",
	        " XXX XXX",
	        "XX XXX X",
	        " XXX XXX",
	        "XX XXX X",
	        " XXX XXX",
	        "XX XXX X",
	        " XXX XXX",
	    },
	    {
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	    },
	    {
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "XXXXX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	    },
	    {
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "XXXXX   ",
	        "   XX   ",
	        "XXXXX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	    },
	    {
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "XXXX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XXXXXXX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XXXXX   ",
	        "   XX   ",
	        "XXXXX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	    },
	    {
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "XXXX XX ",
	        "     XX ",
	        "XXXX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	    },
	    {
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XXXXXXX ",
	        "     XX ",
	        "XXXX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	    },
	    {
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "XXXX XX ",
	        "     XX ",
	        "XXXXXXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "XXXXXXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "XXXXX   ",
	        "   XX   ",
	        "XXXXX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XXXXX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	    },
	    {
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XXXXX",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "XXXXXXXX",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XXXXXXXX",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	    },
	    {
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XXXXX",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XXXXXXXX",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "XXXXXXXX",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	    },
	    {
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XXXXX",
	        "   XX   ",
	        "   XXXXX",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	    },
	    {
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XXX",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	    },
	    {
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XXX",
	        "  XX    ",
	        "  XXXXXX",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "  XXXXXX",
	        "  XX    ",
	        "  XX XXX",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	    },
	    {
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "XXXX XXX",
	        "        ",
	        "XXXXXXXX",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XXXXXXXX",
	        "        ",
	        "XXXX XXX",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	    },
	    {
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XXX",
	        "  XX    ",
	        "  XX XXX",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XXXXXXXX",
	        "        ",
	        "XXXXXXXX",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "XXXX XXX",
	        "        ",
	        "XXXX XXX",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	    },
	    {
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "XXXXXXXX",
	        "        ",
	        "XXXXXXXX",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "XXXXXXXX",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XXXXXXXX",
	        "        ",
	        "XXXXXXXX",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XXXXXXXX",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	    },
	    {
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XXXXXX",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XXXXX",
	        "   XX   ",
	        "   XXXXX",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "   XXXXX",
	        "   XX   ",
	        "   XXXXX",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "  XXXXXX",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	    },
	    {
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "XXXXXXXX",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	        "  XX XX ",
	    },
	    {
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "XXXXXXXX",
	        "   XX   ",
	        "XXXXXXXX",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	    },
	    {
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "XXXXX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "   XXXXX",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	    },
	    {
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	    },
	    {
	        "XXXX    ",
	        "XXXX    ",
	        "XXXX    ",
	        "XXXX    ",
	        "XXXX    ",
	        "XXXX    ",
	        "XXXX    ",
	        "XXXX    ",
	        "XXXX    ",
	        "XXXX    ",
	        "XXXX    ",
	        "XXXX    ",
	        "XXXX    ",
	        "XXXX    ",
	        "XXXX    ",
	        "XXXX    ",
	    },
	    {
	        "    XXXX",
	        "    XXXX",
	        "    XXXX",
	        "    XXXX",
	        "    XXXX",
	        "    XXXX",
	        "    XXXX",
	        "    XXXX",
	        "    XXXX",
	        "    XXXX",
	        "    XXXX",
	        "    XXXX",
	        "    XXXX",
	        "    XXXX",
	        "    XXXX",
	        "    XXXX",
	    },
	    {
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "XXXXXXXX",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        " XXX XX ",
	        "XX XXX  ",
	        "XX XX   ",
	        "XX XX   ",
	        "XX XX   ",
	        "XX XXX  ",
	        " XXX XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        " XXXXX  ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX XXX  ",
	        "XX   XX ",
	        "XX    XX",
	        "XX    XX",
	        "XX    XX",
	        "XX  XXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "XXXXXXX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX      ",
	        "XX      ",
	        "XX      ",
	        "XX      ",
	        "XX      ",
	        "XX      ",
	        "XX      ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "X       ",
	        "XXXXXXX ",
	        " XX XX  ",
	        " XX XX  ",
	        " XX XX  ",
	        " XX XX  ",
	        " XX XX  ",
	        " XX XX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "XXXXXXX ",
	        "XX   XX ",
	        " XX     ",
	        "  XX    ",
	        "   XX   ",
	        "  XX    ",
	        " XX     ",
	        "XX   XX ",
	        "XXXXXXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        " XXXXXX ",
	        "XX XX   ",
	        "XX XX   ",
	        "XX XX   ",
	        "XX XX   ",
	        "XX XX   ",
	        " XXX    ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XXXXX  ",
	        " XX     ",
	        " XX     ",
	        "XX      ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        " XXX XX ",
	        "XX XXX  ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        " XXXXXX ",
	        "   XX   ",
	        "  XXXX  ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        "  XXXX  ",
	        "   XX   ",
	        " XXXXXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "  XXX   ",
	        " XX XX  ",
	        "XX   XX ",
	        "XX   XX ",
	        "XXXXXXX ",
	        "XX   XX ",
	        "XX   XX ",
	        " XX XX  ",
	        "  XXX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "  XXX   ",
	        " XX XX  ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        " XX XX  ",
	        " XX XX  ",
	        " XX XX  ",
	        " XX XX  ",
	        "XXX XXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "   XXXX ",
	        "  XX    ",
	        "   XX   ",
	        "    XX  ",
	        "  XXXXX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        " XX  XX ",
	        "  XXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        " XXXXXX ",
	        "XX XX XX",
	        "XX XX XX",
	        "XX XX XX",
	        " XXXXXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "      XX",
	        "     XX ",
	        " XXXXXX ",
	        "XX  XXXX",
	        "XX XX XX",
	        "XXXX  XX",
	        " XXXXXX ",
	        " XX     ",
	        "XX      ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "   XXX  ",
	        "  XX    ",
	        " XX     ",
	        " XX     ",
	        " XXXXX  ",
	        " XX     ",
	        " XX     ",
	        " XX     ",
	        "  XX    ",
	        "   XXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        " XXXXX  ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "XX   XX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "XXXXXXX ",
	        "        ",
	        "        ",
	        "XXXXXXX ",
	        "        ",
	        "        ",
	        "XXXXXXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "   XX   ",
	        "   XX   ",
	        " XXXXXX ",
	        "   XX   ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "XXXXXXXX",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "  XX    ",
	        "   XX   ",
	        "    XX  ",
	        "     XX ",
	        "    XX  ",
	        "   XX   ",
	        "  XX    ",
	        "        ",
	        " XXXXXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "    XX  ",
	        "   XX   ",
	        "  XX    ",
	        " XX     ",
	        "  XX    ",
	        "   XX   ",
	        "    XX  ",
	        "        ",
	        " XXXXXX ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "    XXX ",
	        "   XX XX",
	        "   XX XX",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	    },
	    {
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "   XX   ",
	        "XX XX   ",
	        "XX XX   ",
	        "XX XX   ",
	        " XXX    ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "   XX   ",
	        "   XX   ",
	        "        ",
	        " XXXXXX ",
	        "        ",
	        "   XX   ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        " XXX XX ",
	        "XX XXX  ",
	        "        ",
	        " XXX XX ",
	        "XX XXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "  XXX   ",
	        " XX XX  ",
	        " XX XX  ",
	        "  XXX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "   XX   ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "   XX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "    XXXX",
	        "    XX  ",
	        "    XX  ",
	        "    XX  ",
	        "    XX  ",
	        "    XX  ",
	        "XXX XX  ",
	        " XX XX  ",
	        " XX XX  ",
	        "  XXXX  ",
	        "   XXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "XX XX   ",
	        " XX XX  ",
	        " XX XX  ",
	        " XX XX  ",
	        " XX XX  ",
	        " XX XX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        " XXX    ",
	        "X  XX   ",
	        "  XX    ",
	        " XX     ",
	        "XX  X   ",
	        "XXXXX   ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        " XXXXX  ",
	        " XXXXX  ",
	        " XXXXX  ",
	        " XXXXX  ",
	        " XXXXX  ",
	        " XXXXX  ",
	        " XXXXX  ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
	    {
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	        "        ",
	    },
		};
		
				
		
		count  = cells.length;
		height = cells[0].length;
		width  = cells[0][0].length();
		
		pixels = new boolean[count][][];
						
		for( int c = 0 ; c < count ; c++ ) {
			
			pixels[c] = new boolean[height][];
								
			for( int y = 0 ; y < height; y++ ) {
				
				pixels[c][y] = new boolean[width];
				
				for( int x = 0 ; x < width ; x++ ) {
					
					if ( cells[c][y].charAt(x) == ' ' ) {
						
						pixels[c][y][x] = false;
						
					} else {
						
						pixels[c][y][x] = true;
						
					}
					
				}
			}
			
		}	
		
		colors = defaultColors;
		
	}
	
		


	
	static final int defaultColors[]  =		// Gives the RGB values for each VGA color byte
	
	{
		
		0x000000,		// 00 Black
		0x0000AB,		// 01 Blue
		0x00AB00,  		// 02 Green
		0x00ABAB,		// 03 Cyan
		0xAB0000,		// 04 Red
		0xAB00AB,		// 05 Violet
		0xA65200,		// 06 Brown
		0xD3D3D3,		// 07 White
		
		0xABABAB,		// 08 Grey
		0x0000FF,		// 09 Bt Blue
		0x00FF00,  		// 10 Bt Green
		0x00FFFF,		// 11 Bt Cyan
		0xFF0000,		// 12 Pink
		0xFF00FF,		// 13 Bt Magenta
		0xFFFF00,		// 14 Yellow
		0xFFFFFF		// 15 White
				
	};
	
	static final String[] colornames = {
		"00 Black",
		"01 Blue" ,
		"02 Green",
		"03 Cyan",
		"04 Red",
		"05 Violet",
		"06 Brown",
		"07 White",
		"08 Grey",
		"09 Bt Blue",
		"10 Bt Green",
		"11 Bt Cyan",
		"12 Pink",
		"13 Bt Magenta",
		"14 Yellow",
		"15 White"
	};
	
		
	
	public void saveFont( File filename) throws IOException {
		
		VgaSoftFont v = this;
		
		BufferedWriter w = new BufferedWriter( new FileWriter(filename ) );
		w.write("# This is a custom font file for the TINY client\r\n");
		w.write("# any line that starts with a '#' or ';' is ignored\r\n");
		w.write("#\r\n");
		w.write("# first is the char height in pixels\r\n");
		w.write( Integer.toString( v.height ));
		w.write("\r\n");
		w.write("#\r\n");		
		w.write("# next is the char width in pixels\r\n");
		w.write( Integer.toString( v.width) );
		w.write("\r\n");
		w.write("#\r\n");		
		w.write("# next comes the color pallet\r\n");
		w.write("# there are 16 text colors\r\n");		
		w.write("# list one color per line\r\n");
		w.write("# each color is a 24 bit hex number\r\n");
		w.write("# the 3 bytes are the red, green, and blue values\r\n");		
		w.write("#\r\n");			
		for(int c=0; c < 16; c++ ) {
			w.write( "# "+colornames[c]+"\r\n");
			
			String s = ((Integer.toHexString(v.colors[c])).toUpperCase());
			
			s = "000000".substring( s.length() ) + s;			// pad left
			
			w.write( s +"\r\n");		
		}
		
		
		w.write("#\r\n");				
		w.write("# finally comes the actual pixel data\r\n");
		w.write("# there must be exactly 256 chars\r\n");				
		w.write("# each char should be (height) lines\r\n");
		w.write("# each of those lines should be (width) chars long\r\n");
		w.write("# and on pixel can be X,x,o,O, or *. Anything else is an off pixel.\r\n");		
		w.write("#\r\n");		
		
		for(int c=0;c<256;c++) {


			w.write("# (ASCII "+c+") \r\n");
				
			
			
			for(int y=0; y<v.height; y++ ){
				
				StringBuffer s = new StringBuffer( v.width );
				
				for( int x=0; x<v.width; x++) {
			
					s.append( v.pixels[c][y][x] ? 'X':'.' );
							
				}
				
				
				w.write( s.toString() +"\r\n");				
				
			}
			
			
		}
		
		w.close();
		
	}
	
	public static void main( String args[] ) throws IOException, FontFormatException {

		System.out.println( "Fontfile converter for TinyClient [http://josh.com/tiny]");
		System.out.println( "This program can read various font file formats and create");
		System.out.println( "a .tf file that TinyClient can load.");
		System.out.println( "");

	
		VgaSoftFont v;
		
		switch (args[0].charAt(0)) {
		
			case 'F': {
				
				System.out.println("Reading FontEdit font from "+args[1]+"...");
				
				v = new VgaSoftFont( new File(args[1]) , FontFileType.FontEdit );				
				
			}
			break;
		
			case 'T': {
				
				System.out.println("Reading TrueType font from "+args[1]+"...");
				
				v = new VgaSoftFont( new File(args[1]) , FontFileType.TTF );				
				
			}
			break;
			
			default: {
				
				System.out.println( "Usage: VgaSoftFont format infile outfile");
				
				System.out.println( "");
				System.out.println( "Where infile is the file to read and outfile is the file to create");
				System.out.println( "");
				System.out.println( "Format can be:");
				System.out.println( "E for file created by FONTEDIT.COM (only works for 16 line fonts)");
				System.out.println( "T for TrueType font file");
				System.out.println( "");
				return;
			}
			
		}
		
		
		System.out.println( "Writing "+args[2]+"...");
		
		v.saveFont(new File(args[2]));
		
		System.out.println( "Done.");
		
					
}	
	
	
}
